import logging

from flask import Blueprint, request
from flask_jwt_extended import jwt_required
from flask_restful import Api, Resource

from portal.database.models import Course
from portal.rest import course_schema, courses_schema, course_import_schema
from portal.service.courses import delete_course, update_course, create_course, \
    update_notes_token, copy_course, filter_course_dump
from portal.service.general import find_course
from portal.service.errors import PortalAPIError, ForbiddenError
from portal.service.permissions import check_client
from portal.service.auth import find_client
from portal.tools.decorators import error_handler
from portal.service import permissions

courses = Blueprint('courses', __name__)
courses_api = Api(courses)

log = logging.getLogger(__name__)


class CourseResource(Resource):
    @error_handler
    @jwt_required
    def get(self, cid):
        client = find_client()
        course = find_course(cid)
        # authorization
        if (check_client(client=client,
                         course=course,
                         permissions=['view_course_full'])):
            return course_schema.dump(course)

        elif (check_client(client=client,
                           course=course,
                           permissions=['view_course_limited'])):
            dump = course_schema.dump(course)
            filtered_course = filter_course_dump(course, dump.data, client)
            return filtered_course

        raise ForbiddenError(uid=client.id)

    @error_handler
    @jwt_required
    def delete(self, cid):
        client = find_client()
        course = find_course(cid)
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        delete_course(course)
        return '', 204

    @error_handler
    @jwt_required
    def put(self, cid):
        client = find_client()
        course = find_course(cid)
        # authorization
        if not (check_client(client=client, course=course, permissions=['update_course'])):
            raise ForbiddenError(uid=client.id)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for course update.')

        data = course_schema.load(json_data)[0]
        update_course(course, data)
        return '', 204


class CourseList(Resource):
    @error_handler
    @jwt_required
    def get(self):
        client = find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        courses_list = Course.query.all()
        return courses_schema.dump(courses_list)

    @error_handler
    @jwt_required
    def post(self):
        client = find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for course creation.')
        data = course_schema.load(json_data)[0]

        new_course = create_course(data)
        return course_schema.dump(new_course)[0], 201


class CourseNotesToken(Resource):
    @error_handler
    @jwt_required
    def get(self, cid):
        client = find_client()
        course = find_course(cid)
        # authorization
        if not (check_client(client=client, course=course,
                             permissions=['handle_notes_access_token'])):
            raise ForbiddenError(uid=client.id)

        return course.notes_access_token

    @error_handler
    @jwt_required
    def put(self, cid):
        client = find_client()
        course = find_course(cid)
        # authorization
        if not (check_client(client=client, course=course,
                             permissions=['handle_notes_access_token'])):
            raise ForbiddenError(uid=client.id)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for notes token update.')

        update_notes_token(course, json_data)
        return '', 204


class CourseImport(Resource):
    @error_handler
    @jwt_required
    def put(self, cid):
        client = find_client()
        course = find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['update_course']):
            raise ForbiddenError(uid=client.id)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for notes token update.')
        data = course_import_schema.load(json_data)[0]
        source_course = find_course(data['source_course'])
        config = data['config']

        copy_course(source_course, course, config)
        return course_schema.dump(course), 204


courses_api.add_resource(CourseResource, '/<string:cid>')
courses_api.add_resource(CourseList, '')
courses_api.add_resource(CourseNotesToken, "/<string:cid>/notes_access_token")
courses_api.add_resource(CourseImport, "/<string:cid>/import")
