import logging

from flask import Blueprint, request
from flask_jwt_extended import jwt_required
from flask_restful import Api, Resource


from portal.rest import role_schema, roles_schema, users_schema, user_list_update_schema, \
    permissions_schema
from portal.service import auth, general
from portal.service.errors import PortalAPIError, ForbiddenError
from portal.service.permissions import check_client
from portal.service.roles import delete_role, update_role, create_role, update_role_permissions, \
    update_users_membership_in_role, add_single_user_to_role, remove_single_user_from_role
from portal.tools.decorators import error_handler

roles = Blueprint('roles', __name__)
roles_api = Api(roles)

log = logging.getLogger(__name__)


class RoleResource(Resource):
    @error_handler
    @jwt_required
    def get(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization
        if not (check_client(client=client, course=course, permissions=['view_course_full'])
                or not check_client(client=client, course=course,
                                    permissions=['view_course_limited'])):
            raise ForbiddenError(uid=client.id)

        return role_schema.dump(role)

    @error_handler
    @jwt_required
    def delete(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization
        if not check_client(client=client, course=course, permissions=['write_roles']):
            raise ForbiddenError(uid=client.id)
        delete_role(role)
        return '', 204

    @error_handler
    @jwt_required
    def put(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization
        if not check_client(client=client, course=course, permissions=['write_roles']):
            raise ForbiddenError(uid=client.id)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for role update.')

        data = role_schema.load(json_data)[0]
        update_role(role, data)
        return '', 204


class RoleList(Resource):
    @error_handler
    @jwt_required
    def get(self, cid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['read_roles']):
            raise ForbiddenError(uid=client.id)

        return roles_schema.dump(course.roles)

    @error_handler
    @jwt_required
    def post(self, cid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['write_roles']):
            raise ForbiddenError(uid=client.id)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for role creation.')

        data = role_schema.load(json_data)[0]

        new_role = create_role(course, data)
        return role_schema.dump(new_role)[0], 201


class RolePermissions(Resource):
    @error_handler
    @jwt_required
    def get(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization
        if not check_client(client=client, course=course, permissions=['read_roles']):
            raise ForbiddenError(uid=client.id)

        return permissions_schema.dump(role.permissions)

    @error_handler
    @jwt_required
    def put(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization
        if not check_client(client=client, course=course, permissions=['write_roles']):
            raise ForbiddenError(uid=client.id)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for update of role permissions.')

        data = permissions_schema.load(json_data)[0]

        update_role_permissions(role, data)
        return '', 204


class RoleUsersList(Resource):
    @error_handler
    @jwt_required
    def get(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization
        if not check_client(client=client, course=course, permissions=['read_roles']):
            raise ForbiddenError(uid=client.id)

        return users_schema.dump(role.users)

    @error_handler
    @jwt_required
    def put(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization
        if not check_client(client=client, course=course, permissions=['write_roles']):
            raise ForbiddenError(uid=client.id)

        json_data = request.get_json()

        data = user_list_update_schema.load(json_data)[0]

        # everything from users_add is added, THEN everything from users_remove is subtracted
        update_users_membership_in_role(role, data)
        return '', 204


class RoleUser(Resource):
    @error_handler
    @jwt_required
    def put(self, cid, rid, uid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        user = general.find_user(uid)
        # authorization
        if not check_client(client=client, course=course, permissions=['write_roles']):
            raise ForbiddenError(uid=client.id)

        add_single_user_to_role(role, user)
        return '', 204

    @error_handler
    @jwt_required
    def delete(self, cid, rid, uid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        user = general.find_user(uid)
        # authorization
        if not check_client(client=client, course=course, permissions=['write_roles']):
            raise ForbiddenError(uid=client.id)

        remove_single_user_from_role(role, user)
        return '', 204


roles_api.add_resource(RoleResource, "/<string:rid>")
roles_api.add_resource(RoleList, "")
roles_api.add_resource(RolePermissions, "/<string:rid>/permissions")
roles_api.add_resource(RoleUsersList, "/<string:rid>/users")
roles_api.add_resource(RoleUser, "/<string:rid>/users/<string:uid>")
