from flask_jwt_extended import get_jwt_identity, get_jwt_claims

from portal.service.errors import IncorrectCredentialsError, UnauthorizedError, PortalAPIError
from portal.service.general import find_component, find_user


def login_user(gitlab_access_token=None, password=None, username=None):
    if gitlab_access_token:
        return auth_gitlab_access_token(
            username=username,
            gitlab_access_token=gitlab_access_token
        )

    return auth_username_password(username=username, password=password)


def auth_gitlab_access_token(username, gitlab_access_token):
    """Authentication using gitlab token

    Steps:
        Send token to gitlab
        Verify that token is for user

    Args:
        username: username of the user attempting to log in
        gitlab_access_token: access token string from gitlab

    Returns: the authenticated user

    """
    if gitlab_access_token is None:
        raise PortalAPIError(400, 'No gitlab access token found.')
    # TODO: validate gitlab token
    user = find_user(username, throws=False)
    if user is None:
        raise IncorrectCredentialsError()
    return user


def auth_username_password(username, password):
    user = find_user(username, throws=False)
    if user is None or password is None:
        raise IncorrectCredentialsError()

    if user.verify_password(password=password):
        return user

    raise IncorrectCredentialsError()


def login_component(name: str, ip_address: str, secret: str):
    client = find_component(name)
    if client.secret != secret:
        raise UnauthorizedError(note="Invalid component secret.")
    if ip_address != client.ip_address:
        raise UnauthorizedError(note="Different source ip address for component login request.")
    return client


def find_client():
    identity = get_jwt_identity()
    claims = get_jwt_claims()
    if not (claims or claims.get('type')):
        raise UnauthorizedError(note="No type claim.")
    if claims['type'] == 'user':
        client = __find_client_helper(identity, 'user', find_user)
    elif claims['type'] == 'component':
        client = __find_client_helper(identity, 'component', find_component)
    else:
        raise UnauthorizedError(note="Invalid jwt claims.")
    return client


def __find_client_helper(identity, resource, method):
    client = method(identity, throws=False)
    if not client:
        raise UnauthorizedError(note=f"{resource.upcase} with identity {identity} not found.")
    return client
