import logging

from portal.database.models import Course, User, ProjectState
from portal.service import general
from portal.service.groups import copy_group
from portal.service.projects import copy_project
from portal.service.roles import copy_role
from portal.tools import time

log = logging.getLogger(__name__)


def copy_course(source: Course, target: Course, config: dict) -> Course:
    if config.get('roles'):
        for role in source.roles:
            copy_role(role, target, with_users=config['roles'])
    if config.get('groups'):
        for group in source.groups:
            copy_group(group, target, with_users=config['groups'])
    if config.get('projects'):
        for project in source.projects:
            copy_project(project, target)

    general.write_entity(target)
    log.info(f"[IMPORT] From {source.id} to:  {target.id}.")

    return target


def delete_course(course: Course):
    log.info(f"[DELETE] Course: {course.id} ({course.codename})")
    general.delete_entity(course)


def update_course(course: Course, data: dict) -> Course:
    course.name = data['name']
    course.codename = data['codename']
    general.write_entity(course)
    log.info(f"[UPDATE] Course {course.id} ({course.codename}): {course}.")
    return course


def create_course(data: dict) -> Course:
    name = data['name']
    codename = data['codename']
    course = Course(name=name, codename=codename)
    general.write_entity(course)
    log.debug(f"[CREATE] Course {course.id}: {course}")
    return course


def update_notes_token(course: Course, json_data: dict) -> Course:
    course.notes_access_token = json_data
    general.write_entity(course)
    log.info(f"[UPDATE] Notes access token ({course.codename}) [{course.id}]: {json_data}")
    return course


def filter_projects_from_course(projects, course):
    def is_active(p):
        return p.state(time.NOW()) == ProjectState.ACTIVE

    active_projects_id = [p.id for p in course.projects if is_active(p)]
    projects = [p for p in projects if p['id'] in active_projects_id]
    return projects


def filter_roles_from_course(roles, course, user):
    user_roles = user.get_roles_in_course(course=course)
    user_roles_id = [r.id for r in user_roles]
    return [r for r in roles if r['id'] in user_roles_id]


def filter_groups_from_course(roles, course, user):
    user_groups = user.get_groups_in_course(course=course)
    user_groups_id = [g.id for g in user_groups]
    return [g for g in roles if g['id'] in user_groups_id]


def filter_course_dump(course: Course, dump: dict, user: User) -> dict:
    log.debug(f"[FILTER] Course: {dump}")
    filtered_dump = {
        'projects': filter_projects_from_course(dump['projects'], course),
        'roles': filter_roles_from_course(dump['roles'], course, user),
        'groups': filter_groups_from_course(dump['groups'], course, user)
    }
    return filtered_dump
