import logging

from portal.database.models import Course, Group, Component, Project, Role, Submission, User
from portal.service.errors import ResourceNotFoundError
from portal import db

log = logging.getLogger(__name__)


def delete_entity(entity):
    log.debug(f"[DELETE] Entity: {entity}")
    db.session.delete(entity)
    db.session.commit()


def write_entity(entity):
    log.debug(f"[WRITE] Entity: {entity}")
    db.session.add(entity)
    db.session.commit()


def find_resource(identifier: str, resource: str, query, throws):
    """Gets an instance of the resource
    Args:
        identifier(str): Id or name or some other identifier of the resource
        resource(str): Resource name
        query: SQLAlchemy query
        throws(bool): True if an exception should be thrown if no entity is found

    Returns: A entity instance

    """
    instance = query.first()

    if not instance:
        log.debug(f"[FIND] Not found {resource.capitalize()}: {identifier}")
        if throws:
            raise ResourceNotFoundError(resource=resource, res_id=identifier)

    log.debug(f"[FIND] Found {resource.capitalize()} for identifier [{identifier}]: {instance}")
    return instance


def find_group(course: Course, identifier: str, throws=True) -> Group:
    query = Group.query.filter_by(course=course).filter(
        (Group.id == identifier) | (Group.name == identifier)
    )

    return find_resource(
        resource='group',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_component(name: str, throws=True) -> Component:
    query = Component.query.filter_by(name=name)
    return find_resource(
        resource='component',
        identifier=name,
        query=query,
        throws=throws
    )


def find_course(identifier: str, throws=True) -> Course:
    query = Course.query.filter(
        ((Course.id == identifier) | (Course.codename == identifier))
    )

    return find_resource(
        resource='course',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_project(course, identifier: str, throws=True) -> Project:
    query = Project.query.filter_by(course=course).filter(
        (Project.id == identifier) | (Project.name == identifier)
    )

    return find_resource(
        resource='project',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_role(course: Course, identifier: str, throws=True) -> Role:
    query = Role.query.filter_by(course=course).filter(
        (Role.id == identifier) | (Role.name == identifier)
    )

    return find_resource(
        resource='role',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_submission(identifier: str, throws=True) -> Submission:
    return find_resource(
        resource='submission',
        identifier=identifier,
        query=Submission.query.filter_by(id=identifier),
        throws=throws
    )


def find_user(identifier: str, throws=True) -> User:
    query = User.query.filter(
        ((User.id == identifier) | (User.username == identifier) | (User.email == identifier))
    )

    return find_resource(
        resource='user',
        identifier=identifier,
        query=query,
        throws=throws
    )