import logging

from portal.database.models import Course, Group, User
from portal.service import general
from portal.service.errors import PortalAPIError
from portal.service.general import find_group, find_course, find_role
from portal.service.users import find_users

log = logging.getLogger(__name__)


def copy_group(source: Group, target: Course, with_users: str):
    new_name = source.name
    count = 1
    if find_group(target, source.name, throws=False):
        new_name += "_copy0"
    while find_group(target, new_name, throws=False):
        new_name[-1] = count
        count += 1
    new_group = Group(target, new_name)
    if with_users == "with_users":
        for user in source.users:
            user.groups.append(new_group)
    return new_group


def delete_group(group: Group):
    log.info(f"[DELETE] Group {group.id} ({group.name})")
    general.delete_entity(group)


def update_group(group: Group, data: dict) -> Group:
    log.info(f"[UPDATE] Group {group.name} to {group}.")
    group.name = data['name']
    general.write_entity(group)
    return group


def create_group(course: Course, data: dict) -> Course:
    new_group = Group(course=course, name=data['name'])
    general.write_entity(new_group)
    log.info(f"[CREATE] Group for course {course.id} ({course.codename}): {new_group}")
    return new_group


def update_user_group_membership(group, data):
    users_old = set(group.users)
    users_add = data.get('add')
    users_remove = data.get('remove')
    users = set()
    if users_add:
        to_add = find_users(data['add'])
        users = users_old.union(to_add)
    if users_remove:
        to_remove = find_users(data['remove'])
        users = users_old.difference(to_remove)
    group.users = list(users)
    general.write_entity(group)
    log.info(f"[UPDATE] Users of group {group.name} to {group.users}.")


def find_users_in_group_by_role(course, group, role_id):
    users = group.users
    if role_id:
        # TODO: Refactor
        # maybe ineffective - use db query instead
        role = find_role(course, role_id)
        users = [user for user in group.users if role in user.roles]
    return users


def add_single_user_to_group(group: Group, user: User) -> Group:
    course = group.course
    if user not in group.users:
        group.users.append(user)
        log.info(
            f"[ADD] Added user {user.username} to group {group.name} in course {course.codename}.")
    else:
        log.info(f"[ADD] User {user.username} is already in group {group.name} "
                 f"in course {course.codename}: no change.")
    return group


def remove_single_user_from_group(group: Group, user: User) -> Group:
    course = group.course
    try:
        group.users.remove(user)
    except ValueError:
        message = f"[REMOVE] Could not remove user " \
                  f"{user.username} from group {group.name} in course " \
                  f"{course.codename}: group does not contain user."
        raise PortalAPIError(400, message=message)
    log.info(f"[REMOVE] User {user.username} from group {group.name}"
             f" in course {course.codename}.")
    return group


def import_group(data: dict, course: Course) -> Group:
    source_course = find_course(data['source_course'])
    source_group = find_group(source_course, data['source_group'])
    new_group = copy_group(source_group, course,
                           data['with_users'])
    general.write_entity(new_group)
    log.info(
        f"[IMPORT] Group {source_group.name} from course "
        f"{source_course.codename} to course {course.codename} "
        f"as {new_group.name}.")
    return new_group
