import logging

import portal.service.courses
import portal.service.general
from portal.database.models import User, Course, Component, Role
from portal.service import general

FILTER_PERMISSION_ATTRS = ['id', 'role_id', 'role', 'description', 'created_at', 'updated_at']

log = logging.getLogger(__name__)


def check_client(client, course, permissions):
    """Checks clients permissions for a course

       Args:
           client(Component): Client instance
           course(Course): Course instance
           permissions(list): List of permissions required

       Returns(bool): True if checks has passed
       """
    check = check_component(component=client, course=course, permissions=permissions) or \
            check_user(user=client, course=course, permissions=permissions)

    return check


def check_component(component: Component, permissions: list, course: Course = None) -> bool:
    """Checks component permissions for a course

    Args:
        component(Component): Component instance
        course(Course): Course instance
        permissions(list): List of permissions required

    Returns(bool): True if checks has passed

    """
    # TODO
    if not isinstance(component, Component):
        return False
    return True


def check_user(user: User, course: Course, permissions: list) -> bool:
    """Checks a user's permissions based on his roles in the course

    Args:
        user(User): User instance
        course(Course): Course instance
        permissions(list): Permissions list

    Returns(bool): True if checks pass

    """
    log.debug(f"[PERM] User {user.username} in course {course.codename}: {permissions}")
    if not isinstance(user, User):
        return False
    if check_sysadmin(user):
        return True

    effective_permissions = effective_permissions_for_user(user, course=course)

    res = all(effective_permissions.get(p, False) for p in permissions)
    log.debug(f"[PERM] {permissions} for {user.username}: {res}")
    return res


def check_sysadmin(user: User) -> bool:
    """Checks whether the user is sysadmin

    Args:
        user(User): User instance

    Returns(bool): True if checks pass
    """
    if not isinstance(user, User):
        return False
    return user.is_admin


def build_effective_permissions(user: User, *courses):
    result = {}
    for course in courses:
        result[course.id] = effective_permissions_for_user(user, course=course)
    return result


def get_effective_permissions(user: User, course_id: str) -> dict:
    course = portal.service.general.find_course(course_id, throws=False)
    courses = [course] if course else user.courses
    return build_effective_permissions(user, *courses)


def effective_permissions_for_user(user: User, course: Course):
    permissions = user.get_permissions_for_course(course)
    result = {}
    for permission in permissions:
        for key, value in vars(permission).items():
            if not key.startswith("_") and key not in FILTER_PERMISSION_ATTRS:
                result[key] = result.get(key) or value
    log.debug(
        f"[PERM] Effective permissions: {user.username} in course {course.codename}: {result}"
    )
    return result
