import logging

import portal.service
from portal.database import SubmissionState
from portal.database.models import Project, Course, User, Submission
from portal.service import general
from portal.service.general import find_project

log = logging.getLogger(__name__)


def copy_project(source: Project, target: Course):
    new_name = source.name
    count = 1
    if find_project(target, source.name, throws=False):
        new_name += "_copy0"
    while find_project(target, new_name, throws=False):
        new_name[-1] = count
        count += 1
    new_project = Project(target, new_name, source.config.test_files_source)
    new_project.set_config(**vars(source.config))
    return new_project


def can_create_submission(user: User, project: Project) -> bool:
    """Checks if a user may create a new submission.

    A user is permitted to create a new submission if there are no submissions
    created or queued for him in the given project.

    Args:
        user(User): the user who wants to crete a new submission
        project(Project): the project in which the user wants to create a submission

    Returns: whether the user can create a new submission in the project

    """
    conflicting_states = [SubmissionState.CREATED, SubmissionState.READY, SubmissionState.QUEUED]
    conflict = Submission.query.filter(Submission.user == user) \
        .filter(Submission.project == project) \
        .filter(Submission.state.in_(conflicting_states)) \
        .first()

    return conflict is None


def delete_project(project):
    log.info(f"[DELETE] Project {project.id} ({project.name}).")
    general.delete_entity(project)


def update_project(project, data):
    project.name = data['name']
    general.write_entity(project)
    log.info(f"[UPDATE] Project {project.id} ({project.name}): {project}.")


def create_project(course, data):
    new_project = Project(course=course, name=data['name'],
                          test_files_source=data['config']['test_files_source'])
    general.write_entity(new_project)
    log.info(f"[CREATE] Project for course {course.id} ({course.codename}): "
             f"{new_project}")
    return new_project


def update_project_config(project: Project, data: dict):
    course = project.course
    project.set_config(**data)
    general.write_entity(project)
    log.info(f"[UPDATE] Configuration for project {project.id} in '"
             f"course {course.id} to {data}.")
    return project.config


def query_submissions(course, project, user):
    return Submission.query.filter(Submission.user_id == user.id) \
        .filter(Submission.project_id == project.id) \
        .join(Course, Course.id == project.course_id).filter(Course.id == course.id)


def find_project_submissions(course, project, user_id):
    submissions = project.submissions
    if user_id:
        user = portal.service.general.find_user(user_id)
        submissions = query_submissions(course, project, user)
    return submissions
