import logging

from portal.database.models import Course, Role
from portal.service.errors import PortalAPIError
from portal.service.general import find_role, delete_entity, write_entity
from portal.service.users import find_users

log = logging.getLogger(__name__)


def copy_role(source: Role, target: Course, with_users: str):
    new_name = source.name
    count = 1
    if find_role(target, source.name, throws=False):
        new_name += "_copy0"
    while find_role(target, new_name, throws=False):
        new_name[-1] = count
        count += 1
    new_role = Role(target, new_name)
    new_role.set_permissions(**vars(source.permissions))
    new_role.description = source.description
    if with_users == "with_users":
        for user in source.users:
            user.roles.append(new_role)
    return new_role


def delete_role(role):
    delete_entity(role)


def update_role(role, data):
    role.description = data['description']
    role.name = data['name']
    write_entity(role)
    log.info(f"[UPDATE] Role {role.id} to {role}.")


def create_role(course, data):
    new_role = Role(course=course, name=data['name'])
    new_role.description = data.get('description') or ""
    write_entity(new_role)
    log.info(f"[CREATE] Role for course {course.id}: {new_role}")
    return new_role


def update_role_permissions(role, data):
    role.set_permissions(**data)
    log.info(f"[UPDATE] Permissions for role {role.id} "
              f"in course {role.course.id}: {data}.")


def update_users_membership_in_role(role, data):
    users_old = set(role.users)
    users_add = data.get('add')
    users_remove = data.get('remove')
    users = set()
    if users_add:
        to_add = find_users(data['add'])
        users = users_old.union(to_add)
    if users_remove:
        to_remove = find_users(data['remove'])
        users = users_old.difference(to_remove)
    role.users = list(users)
    write_entity(role)
    log.info(f"[UPDATE] Users in Role {role.id}: {data}.")


def add_single_user_to_role(role, user):
    if user not in role.users:
        role.users.append(user)
        log.info(f"[ADD] User {user.id} to role "
                 f"{role.id} in course {role.course.id}.")
    else:
        log.info(f"[ADD] User {user.id} is already "
                 f"in role {role.id} in course {role.course.id}: no change.")


def remove_single_user_from_role(role, user):
    course = role.course
    try:
        role.users.remove(user)
    except ValueError:
        raise PortalAPIError(400, message=f"Could not remove user {user.id} "
                                          f"from role {role.id} in course {course.id}: "
                                          f"role does not contain user.")
    log.info(f"[REMOVE] User {user.id} from role {role.id} in course {course.id}.")
