import logging

from portal import storage
from portal.database.models import Submission, User, Project
from portal.service.errors import ForbiddenError
from portal.service.general import write_entity, delete_entity

log = logging.getLogger(__name__)


def create_submission(user: User, project: Project, file_params,
                      project_params_string) -> Submission:
    """Creates a new submission in the database and downloads its files.

    Zip and git sources are processed differently.

    Args:
        user(User): the submission's author
        project(Project): the project the submission falls under
        file_params(dict): parameters for Storage - how and where to download files
        project_params_string(str): custom parameters specified by project with their values as a JSON-encoded string

    Returns(Submission): the created submission entity

    """
    # adding review at submission create is not supported
    # TODO: store all configuration relevant for submission scheduling:
    # project.submissions_allowed from, to, archive;
    # project.config.submission_scheduler_config, submission_params?
    new_submission = Submission(user=user, project=project, parameters=project_params_string)
    write_entity(new_submission)

    # download relevant files into Storage
    file_params['whitelist'] = project.config.file_whitelist
    if file_params['source']['type'] == 'zip':
        file_params['source']['path'] = "todo"  # TODO: find out how to download files through flask
    storage.submissions.create(new_submission.id, **file_params)
    return new_submission


def copy_submission(source: Submission, note: str) -> Submission:
    new_submission = Submission(user=source.user, project=source.project,
                                parameters=source.parameters)
    new_submission.note = note
    write_entity(new_submission)
    storage.submissions.clone(source.id, new_submission.id)
    return new_submission


def delete_submission(submission: Submission):
    log.info(f"[DELETE] Submission {submission.id}")
    delete_entity(submission)


def update_submission_state(client, data, full, submission):
    new_state = data['state']
    from portal.database import SubmissionState
    if new_state != SubmissionState.CANCELLED and not full:
        raise ForbiddenError(uid=client.id,
                             note=f"User {client.id} cannot update "
                                  f"state to other than CANCELLED.")
    submission.change_state(new_state)
    write_entity(submission)
    log.info(f"[UPDATE] Submission State {submission.id} to {submission.state}")
