import logging

from portal.database.models import User, Course, Group, Submission, Project
from portal.service import general
from portal.service.errors import PortalAPIError, IncorrectCredentialsError
from portal.service.general import find_course, find_user

log = logging.getLogger(__name__)


def create_user(data: dict) -> User:
    admin = data.get('is_admin', False)
    new_user = User(uco=data['uco'], email=data['email'], username=data['username'],
                    is_admin=admin)
    general.write_entity(new_user)
    log.info(f"[CREATE] User: {new_user}")
    return new_user


def update_user(user: User, data: dict, full=False) -> User:
    user.email = data['email']
    user.uco = data['uco']
    if data.get('name') is not None:
        user.name = data['name']
    if (data.get("is_admin") is not None) and full:
        user.is_admin = data['is_admin']
    general.write_entity(user)
    log.info(f"[UPDATE] user {user.id} to {user}.")
    return user


def update_user_password(client: User, data: dict, user: User) -> User:
    if not data.get('new_password'):
        raise PortalAPIError(400, 'missing new password at password change')
    if client == user and not user.is_admin:
        if not data.get('old_password'):
            raise PortalAPIError(400, 'missing old password for password update')
        old_password = data['old_password']
        if not user.verify_password(old_password):
            raise IncorrectCredentialsError()
    user.set_password(data['new_password'])
    general.write_entity(user)
    return user


def delete_user(user: User):
    log.debug(f"[DELETE] User(user.username): {user.id} .")
    general.delete_entity(user)


def find_users(ids: list) -> list:
    return [find_user(i) for i in ids]


def find_users_filtered(course_id: str, group_id: str):
    users = User.query
    if course_id:
        users = users.join(User.roles) \
            .join(Course).filter((Course.id == course_id) | (Course.codename == course_id))
    if group_id:
        users = users.join(User.groups).filter((Group.id == group_id) | (Group.name == group_id))

    return users


def find_groups_filtered(user: User, course_id: str):
    groups = user.groups
    if course_id:
        groups = user.get_groups_in_course(find_course(course_id))
    return groups


def find_roles_filtered(user: User, course_id: str):
    # not using db query to filter yet... user does not have that many roles
    roles = user.roles
    if course_id:
        roles = user.get_roles_in_course(find_course(course_id))
    return roles


def find_reviews(user: User):
    reviews = []
    for item in user.review_items:
        if item.review not in reviews:
            reviews.append(item.review)
    return reviews


def find_submissions_filtered(user: User, course_id: str, project_id: str):
    submissions = Submission.query.filter_by(user=user)
    # user.submissions cannot be used as the base query
    if course_id:
        submissions = submissions.join(Project)
        if project_id:
            submissions = submissions.filter(
                (Project.id == project_id) | (Project.name == project_id))
        submissions = submissions.join(Course).filter(
            (Course.id == course_id) | (Course.codename == course_id))
    return submissions
